The TSQL of Text Files

--------------------------------------------------------------------
IF OBJECT_ID (N'dbo.ListPaths') IS NOT NULL
   DROP PROCEDURE dbo.ListPaths
GO
CREATE PROCEDURE dbo.ListPaths
@FileSpec VARCHAR(2000),
@order VARCHAR (80) = '/O-D'--sort by date time oldest first
/*
produce a table with a single column consisting of the path of every file, including subdirectories, of the directory specified You can specify the order in the order parameter
             N  By name (alphabetic)       S  By size (smallest first)
             E  By extension (alphabetic)  D  By date/time (oldest first)
             G  Group directories first    -  Prefix to reverse order

*/
AS
DECLARE @myfiles TABLE (MyID INT IDENTITY(1,1) PRIMARY KEY, FullPath VARCHAR(2000))
DECLARE @CommandLine VARCHAR(4000)
IF @order IS NOT NULL -- abort if the order is silly
   BEGIN
   SELECT @CommandLine =LEFT('dir "' + @FileSpec + '" /A-D /B /S '+@order,4000)
   /*
     /A          Displays files with specified attributes.
     attributes   D  Directories                R  Read-only files
                  H  Hidden files               A  Files ready for archiving
                  S  System files               -  Prefix meaning not
     /B          Uses bare format (no heading information or summary).
     /S          Displays files in specified directory and all subdirectories.
   */
   INSERT INTO @MyFiles (FullPath)
       EXECUTE xp_cmdshell @CommandLine
   DELETE FROM @MyFiles WHERE fullpath IS NULL OR fullpath='File Not Found'
   END
SELECT fullpath FROM @MyFiles
-------------------------------------------------------------
GO
EXECUTE listpaths 'C:\Workbench\*.txt' 
Fine, but there are problems. Try capturing the result into a table, ... 

DECLARE @files TABLE (MyID INT IDENTITY(1,1) PRIMARY KEY, [Path] VARCHAR(2000))
INSERT INTO @files(path)
   EXECUTE listpaths 'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\LOG'

  

--------------------------------------------------------------------
IF OBJECT_ID (N'dbo.ListPathsXML') IS NOT NULL
   DROP PROCEDURE dbo.ListPathsXML
GO
CREATE PROCEDURE dbo.ListPathsXML
@FileSpec VARCHAR(2000),
@order VARCHAR (80) = '/O-D',--sort by date time oldest first
@xmlFileList XML OUTPUT

AS
DECLARE @myfiles TABLE (MyID INT IDENTITY(1,1) PRIMARY KEY, FullPath VARCHAR(2000))
DECLARE @CommandLine VARCHAR(4000)
IF @order IS NOT NULL -- abort if the order is silly
   BEGIN
   SELECT @CommandLine =LEFT('dir "' + @FileSpec + '" /A-D /B /S '+@order,4000)
   INSERT INTO @MyFiles (FullPath)
       EXECUTE xp_cmdshell @CommandLine
   DELETE FROM @MyFiles WHERE fullpath IS NULL
       OR fullpath = 'File Not Found'
   END
SET @xmlFileList = (SELECT fullpath FROM @MyFiles
                         FOR
                          XML PATH('thefile'),
                              ROOT('thefiles'),
                              TYPE)
GO



DECLARE @FileList XML
EXECUTE ListPathsXML 'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\LOG',
    DEFAULT , @XMLFileList = @FileList OUTPUT
SELECT   x.thefile.value('fullpath[1]', 'varchar(2000)') AS [path]
        FROM    @FileList.nodes('//thefiles/thefile') AS x ( thefile )
------------------------------------------------------------------------ 


DECLARE @FileList XML, @FilePath VARCHAR(2000)
SELECT @FilePath='C:\Program Files\Microsoft SQL Server'
EXECUTE spGetFilePaths @FilePath, '*.*',
    @subdirectories = >1, @XMLFileList = @FileList OUTPUT
SELECT directoryList.* FROM  
       (SELECT  @filepath AS BaseDirectory,
           x.thefile.value('thePath[1]', 'varchar(200)') AS Fullpath,
            x.thefile.value('filename[1]', 'varchar(200)') AS [filename],
            x.thefile.value('path[1]', 'varchar(200)') AS [Subdirectory]
        FROM    @FileList.nodes('//thefiles/thefile') AS x ( thefile ))directoryList 

/*
This is a handy routine for getting a list of files and directories (including all subdirectories) into a table or XML list. I couldn't decide whether to Blog it or keep it a secret. I compromised by blogging here! It needs SQL Server 2005, but a similar routine would work in SQL Server 200. You'll need to allow the use of xp_cmdshell too.

There is a problem with using the Scripting.FileSystemObject for doing this, in that, in TSQL, there is no way of iterating through a COM collection other than getting each member via its index. For some cloth-headed reason lost in the mists of time, Microsoft forgot to implement this in the Files collection of the folder object. This means that you have to know the name of the file before you can find out its name. Hmm.....

The only way to do it is 

    Set oFolder = oFs.GetFolder(FullPath)

    For Each oFile In oFolder.Files
      print oFile.Name
    Next

which you can't do in TSQL with the sp_OAMethod!

I had to use the XP_CmdShell method of getting this information. you may think that walking the subdirectories to get the files might be a recursive task, but it is actually almost as easy doing it by iteration. 
*/

ALTER PROCEDURE [dbo].[spGetFilePaths]
   @BaseDirectory VARCHAR(255),-- the initial directory e.g. 'c:\mypath' 
   @filespec VARCHAR(10) = '*.*',--the files you want 
   @wanted VARCHAR(10) = 'files',--files or directories 
   @subdirectories INT = 1,--do we want the subdirectories too 
   @xmlFileList XML OUTPUT-- we are essentially passing back a table 

/*
This procedure returns an XML list of all the files or directories to the spec you determine, using xp_CMDShell, which needs to be allowed.  A system administrator can enable the use of 'xp_cmdshell' by using sp_configure. For more information about enabling 'xp_cmdshell', see "Surface Area Configuration" in SQL Server Books Online. 

to use   spGetFilePaths...
e.g. 
DECLARE @FileList XML 
EXECUTE spGetFilePaths 'f:pentlow\work\programs\SQL\Templates\ssc', '*.*', 
    @subdirectories = 1, @XMLFileList = @FileList OUTPUT 
SELECT @filelist 
*/
AS 
   SET NOCOUNT ON 
/* declare our variables ...*/
   DECLARE @DirectoryName VARCHAR(255) 
   DECLARE @Directory_ID INT 
   DECLARE @command VARCHAR(255) 
/* ..and our local tables */
   DECLARE @directory TABLE
      (
       Directory_ID INT IDENTITY(1, 1),
       directory VARCHAR(255),
       done INT
      ) 
   DECLARE @line TABLE (line VARCHAR(255)) 
   DECLARE @files TABLE
      (
       files_ID INT IDENTITY(1, 1),
       thepath VARCHAR(255),
       [Filename] VARCHAR(100),
       [Path] VARCHAR(255)
      )
--take off any final \
SELECT @BaseDirectory=LTRIM(RTRIM(@BaseDirectory)) 
IF SUBSTRING(REVERSE(@BaseDirectory),1,1)='\'
   SELECT @BaseDirectory
           =SUBSTRING(@BaseDirectory,1,LEN(@BaseDirectory)-1)

/* now loop around getting the files */
   INSERT   INTO @directory
            (directory, done)
            SELECT   @baseDirectory, 0 

   WHILE 1 = 1 
      BEGIN 
         DELETE   FROM @line 
         SELECT TOP 1
                  @Directory_ID = directory_ID, @DirectoryName = directory
         FROM     @directory
         WHERE    done = 0 
         IF @@rOWCOUNT = 0 
            BREAK ; 
--get the files first
         SELECT   @command = 'dir "' + @directoryname + '\' + @Filespec + '" /A-D /B' 
         INSERT   INTO @line
                  EXECUTE xp_cmdshell @command 
         IF NOT EXISTS ( SELECT  line
                         FROM    @line
                         WHERE   line LIKE 'File Not Found' ) 
            INSERT   INTO @files
                     (thePath, [filename], [path])
                     SELECT   @directoryname + '\' + line, line, @DirectoryName
                     FROM     @line
                     WHERE    line IS NOT NULL
--get the directories 
         DELETE   FROM @line 
         SELECT   @command = 'dir "' + @directoryname + '" /AD  /B' 
         INSERT   INTO @line
                  EXECUTE xp_cmdshell @command 
         INSERT   INTO @directory
                  (directory, done)
                  SELECT   @directoryname + '\' + line, 0
                  FROM     @line
                  WHERE    line IS NOT NULL
         UPDATE   @directory
         SET      done = -1
         WHERE    directory_ID = @directory_ID 
         IF @subdirectories = 0 
            BREAK 
      END 
   IF @wanted = 'files' 
      SET @xmlFileList = (SELECT thePath, [filename], [path]
                          FROM   @files 
                         FOR
                          XML PATH('thefile'),
                              ROOT('thefiles'),
                              TYPE
                         ) 
   ELSE 
      SET @xmlFileList = (SELECT thepath = directory FROM @directory 
                         FOR
                          XML PATH('thefile'),
                              ROOT('thefiles'),
                              TYPE
                         ) 

--======================================================

--!! Accessing the SHELL.APPLICATION to get a directory listing. 

--list all subdirectories  beginning with M from "c:\program files" */
SELECT [path] FROM dbo.dir('c:\program files')
       WHERE name LIKE 'm%' AND IsFolder =1
--list every file and subdirectory in the WINNT directory
SELECT  * FROM dbo.dir('c:\winnt') 

SELECT SUM(size), COUNT(*)
FROM dbo.files ('C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\LOG', 1,1)

Getting Text into a TSQL Variable 
Now we have solved the awful problem of getting the names of the files we want, we can have some fun. It is now easy to read a file into SQL Server. There are a number of ways of getting stuff in to SQL Server but this is the simplest. 

DECLARE @LotsOfText VARCHAR(MAX)
SELECT  @LotsOfText = BulkColumn
FROM    OPENROWSET(BULK 'C:\workbench\MyBigfile.txt', SINGLE_BLOB) AS x   


DECLARE @LotsOfText NVARCHAR(MAX),
       @ii INT,
       @iiMax INT,
       @File VARCHAR(2000),
       @Command NVARCHAR(4000)
DECLARE @files TABLE (MyID INT IDENTITY(1,1) PRIMARY KEY, [Path] VARCHAR(2000))

DECLARE @FileList XML
EXECUTE ListPathsXML 'C:\MyDocuments\MyData.TXT',
    DEFAULT , @XMLFileList = @FileList OUTPUT

INSERT INTO @files(path)
   SELECT   x.thefile.value('fullpath[1]', 'varchar(2000)') AS [path]
        FROM    @FileList.nodes('//thefiles/thefile') AS x ( thefile )
--don't look at the current errorlog!
SELECT @ii=1, @iiMax=MAX(MyID) FROM @Files
WHILE @ii<=@iiMax
   BEGIN
   SELECT @File= [path] FROM @files WHERE MyID=@ii
   SELECT @command='SELECT  @LotsOfText = BulkColumn
FROM    OPENROWSET(BULK '''+REPLACE(@FILE,'''','''''')+''', SINGLE_BLOB) AS x'
   EXECUTE sp_ExecuteSQL @command, N'@lotsOfText nvarchar(max) output ',@lotsoftext output
   IF PATINDEX('%Explosive%',@LotsOfText)>0
       SELECT 'Error report found in file "'+@file+'"'
   SELECT @ii=@ii+1
   END
SELECT CAST(@ii AS VARCHAR(5))+' files checked' 

--!! Getting Text into a Table 

DECLARE @LotsOfText NVARCHAR(MAX),
       @ii INT,
       @iiMax INT,
       @File VARCHAR(2000)
DECLARE @files TABLE (MyID INT IDENTITY(1,1) PRIMARY KEY, [Path] VARCHAR(2000))
DECLARE @lines TABLE (MyID INT IDENTITY(1,1) PRIMARY KEY, [line] NVARCHAR(MAX))

DECLARE @FileList XML
EXECUTE ListPathsXML 'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\LOG\ERRORLOG.*',
    DEFAULT , @XMLFileList = @FileList OUTPUT

INSERT INTO @files(path)
   SELECT   x.thefile.value('fullpath[1]', 'varchar(2000)') AS [path]
        FROM    @FileList.nodes('//thefiles/thefile') AS x ( thefile )
DELETE FROM @files WHERE REVERSE(path) LIKE 'golrorre%'
--don't look at the current errorlog!
SELECT @ii=1, @iiMax=MAX(MyID) FROM @Files
WHILE @ii<=@iiMax
   BEGIN
   SELECT @File= [path] FROM @files WHERE MyID=@ii
   INSERT INTO @lines(line)
       EXECUTE spLoadTextFromAFile @file, @Unicode=1
   SELECT @ii=@ii+1
   END
SELECT MyID AS [line Number], Line, @file FROM @lines
       WHERE line LIKE '%Error%' 
Write data from a TSQL variable into a file 
If you want to write text out from a variable to a file, here is something that will do the trick. 

-----------------------------------------------------------------
IF OBJECT_ID (N'dbo.spSaveTextToFile') IS NOT NULL
   DROP PROCEDURE dbo.spSaveTextToFile
GO
CREATE PROCEDURE spSaveTextToFile
  @TheString VARCHAR(MAX),
  @Filename VARCHAR(255),
  @Unicode INT=0
AS
  SET NOCOUNT ON
  DECLARE @MySpecialTempTable VARCHAR(255)
  DECLARE @Command NVARCHAR(4000)
  DECLARE @RESULT INT

--firstly we create a global temp table with a unique name
  SELECT  @MySpecialTempTable = '##temp'
       + CONVERT(VARCHAR(12), CONVERT(INT, RAND() * 1000000))
--then we create it using dynamic SQL, & insert a single row
--in it with the MAX Varchar stocked with the string we want
  SELECT  @Command = 'create table ['
       + @MySpecialTempTable
       + '] (MyID int identity(1,1), Bulkcol varchar(MAX))
insert into ['
       + @MySpecialTempTable
       + '](BulkCol) select @TheString'
  EXECUTE sp_ExecuteSQL @command, N'@TheString varchar(MAX)',
           @TheString

--then we execute the BCP to save the file
  SELECT  @Command = 'bcp "select BulkCol from ['
          + @MySpecialTempTable + ']'
          + '" queryout '
          + @Filename + ' '
         + CASE WHEN @Unicode=0 THEN '-c' ELSE '-w' END
          + ' -T -S' + @@servername
  EXECUTE @RESULT= MASTER..xp_cmdshell @command, NO_OUTPUT
  EXECUTE ( 'Drop table ' + @MySpecialTempTable )
  RETURN @result
GO 

DECLARE @LotsOfText VARCHAR(MAX)
SELECT  @LotsOfText = BulkColumn
FROM    OPENROWSET(BULK 'C:\workbench\Moby-Dick.txt', SINGLE_BLOB) AS x  
EXECUTE spSaveTextToFile @lotsOfText, 'C:\workbench\Moby-Dick-copy.txt',0,0 
Write the String-based results of a SQL Expression into a file 
We can do other things with the same idea we've just seen. The following procedure even allows you to write the results of a SQL Statement to a file as long as the result is a single string column. This sort of idea can be adapted to a number of purposes. 

IF OBJECT_ID (N'dbo.spSaveTextResultToFile') IS NOT NULL
   DROP PROCEDURE dbo.spSaveTextResultToFile
GO PROCEDURE spSaveTextResultToFile
  @TheSQL VARCHAR(MAX),
  @Filename VARCHAR(255),
  @Unicode INT=0
/*
e.g. spSaveTextResultToFile 'Select logstring+'', ''+convert(char(11),insertionDate,113) from activitylog', 'C:\workbench\Logreport.txt'

*/
AS
  SET NOCOUNT ON
  DECLARE @MySpecialTempTable VARCHAR(255)
  DECLARE @Command NVARCHAR(4000)
  DECLARE @RESULT INT

  IF CHARINDEX ('Select ',LTRIM(@TheSQL))=0
       BEGIN
       RAISERROR ('Usage spSaveTextResultToFile <The SQL Expression> <The Filename)',16,1)
       RETURN 1
       END
--firstly we create a global temp table with a unique name
  SELECT  @MySpecialTempTable = '##temp'
       + CONVERT(VARCHAR(12), CONVERT(INT, RAND() * 1000000))
--then we create it using dynamic SQL,
--
  SELECT  @Command = 'create table ['
       + @MySpecialTempTable
       + '] (MyID int identity(1,1), MyLine varchar(MAX))
insert into ['
       + @MySpecialTempTable
       + '](MyLine) ' +@TheSQL
  EXECUTE sp_ExecuteSQL @command

--then we execute the BCP to save the file
  SELECT  @Command = 'bcp "select Myline from ['
          + @MySpecialTempTable + ']'
          + '" queryout '
          + @Filename
         + CASE WHEN @Unicode=0 THEN '-c' ELSE '-w' END
          + ' -T -S' + @@servername
  EXECUTE @RESULT= MASTER..xp_cmdshell @command, NO_OUTPUT
  EXECUTE ( 'Drop table ' + @MySpecialTempTable ) 

ALTER PROCEDURE [dbo].[spGetFilePaths]
   @BaseDirectory VARCHAR(255),-- the initial directory e.g. 'c:\mypath' 
   @filespec VARCHAR(10) = '*.*',--the files you want 
   @wanted VARCHAR(10) = 'files',--files or directories 
   @subdirectories INT = 1,--do we want the subdirectories too 
   @xmlFileList XML OUTPUT-- we are essentially passing back a table 

/*
This procedure returns an XML list of all the files or directories to the spec you determine, using xp_CMDShell, which needs to be allowed.  A system administrator can enable the use of 'xp_cmdshell' by using sp_configure. For more information about enabling 'xp_cmdshell', see "Surface Area Configuration" in SQL Server Books Online. 

to use   spGetFilePaths...
e.g. 
DECLARE @FileList XML 
EXECUTE spGetFilePaths 'f:pentlow\work\programs\SQL\Templates\ssc', '*.*', 
    @subdirectories = 1, @XMLFileList = @FileList OUTPUT 
SELECT @filelist 
*/
AS 
   SET NOCOUNT ON 
/* declare our variables ...*/
   DECLARE @DirectoryName VARCHAR(255) 
   DECLARE @Directory_ID INT 
   DECLARE @command VARCHAR(255) 
/* ..and our local tables */
   DECLARE @directory TABLE
      (
       Directory_ID INT IDENTITY(1, 1),
       directory VARCHAR(255),
       done INT
      ) 
   DECLARE @line TABLE (line VARCHAR(255)) 
   DECLARE @files TABLE
      (
       files_ID INT IDENTITY(1, 1),
       thepath VARCHAR(255),
       [Filename] VARCHAR(100),
       [Path] VARCHAR(255)
      )
--take off any final \
SELECT @BaseDirectory=LTRIM(RTRIM(@BaseDirectory)) 
IF SUBSTRING(REVERSE(@BaseDirectory),1,1)='\'
   SELECT @BaseDirectory
           =SUBSTRING(@BaseDirectory,1,LEN(@BaseDirectory)-1)

/* now loop around getting the files */
   INSERT   INTO @directory
            (directory, done)
            SELECT   @baseDirectory, 0 

   WHILE 1 = 1 
      BEGIN 
         DELETE   FROM @line 
         SELECT TOP 1
                  @Directory_ID = directory_ID, @DirectoryName = directory
         FROM     @directory
         WHERE    done = 0 
         IF @@rOWCOUNT = 0 
            BREAK ; 
--get the files first
         SELECT   @command = 'dir "' + @directoryname + '\' + @Filespec + '" /A-D /B' 
         INSERT   INTO @line
                  EXECUTE xp_cmdshell @command 
         IF NOT EXISTS ( SELECT  line
                         FROM    @line
                         WHERE   line LIKE 'File Not Found' ) 
            INSERT   INTO @files
                     (thePath, [filename], [path])
                     SELECT   @directoryname + '\' + line, line, @DirectoryName
                     FROM     @line
                     WHERE    line IS NOT NULL
--get the directories 
         DELETE   FROM @line 
         SELECT   @command = 'dir "' + @directoryname + '" /AD  /B' 
         INSERT   INTO @line
                  EXECUTE xp_cmdshell @command 
         INSERT   INTO @directory
                  (directory, done)
                  SELECT   @directoryname + '\' + line, 0
                  FROM     @line
                  WHERE    line IS NOT NULL
         UPDATE   @directory
         SET      done = -1
         WHERE    directory_ID = @directory_ID 
         IF @subdirectories = 0 
            BREAK 
      END 
   IF @wanted = 'files' 
      SET @xmlFileList = (SELECT thePath, [filename], [path]
                          FROM   @files 
                         FOR
                          XML PATH('thefile'),
                              ROOT('thefiles'),
                              TYPE
                         ) 
   ELSE 
      SET @xmlFileList = (SELECT thepath = directory FROM @directory 
                         FOR
                          XML PATH('thefile'),
                              ROOT('thefiles'),
                              TYPE
                         ) 

/*===================================================*/
IF OBJECT_ID (N'dbo.Dir') IS NOT NULL
   DROP FUNCTION dbo.Dir
GO
CREATE FUNCTION [dbo].[Dir](@Wildcard VARCHAR(8000))
/* returns a table representing all the items in a folder. It takes as parameter the path to the folder. It does not take wildcards in the same way as a DIR command. Instead, you would be expected to filter the results of the function using SQL commands
Notice that the size of the item (e.g. file) is not returned by this function. 

This function uses the Windows Shell COM object via OLE automation. It opens a folder and iterates though the items listing their relevant properties. You can use the SHELL object to do all manner of things such as printing, copying, and moving filesystem objects, accessing the registry and so on. Powerful medicine.

--e.g.
--list all subdirectories directories beginning with M from "c:\program files"
SELECT [path] FROM dbo.dir('c:\program files') 
       WHERE name LIKE 'm%' AND IsFolder =1
SELECT  * FROM dbo.dir('C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\LOG')

*/
RETURNS @MyDir TABLE 
(
    -- columns returned by the function
       [name] VARCHAR(2000),    --the name of the filesystem object
       [path] VARCHAR(2000),    --Contains the item's full path and name. 
       [ModifyDate] DATETIME,   --the time it was last modified 
       [IsFileSystem] INT,      --1 if it is part of the file system
       [IsFolder] INT,          --1 if it is a folsdder otherwise 0
       [error] VARCHAR(2000)    --if an error occured, gives the error otherwise null
)
AS
-- body of the function
BEGIN
   DECLARE 
       --all the objects used
       @objShellApplication INT, 
       @objFolder INT,
       @objItem INT,
       @objErrorObject INT,
       @objFolderItems INT, 
       --potential error message shows where error occurred.
       @strErrorMessage VARCHAR(1000), 
       --command sent to OLE automation
       @Command VARCHAR(1000), 
       @hr INT, --OLE result (0 if OK)
       @count INT,@ii INT,
       @name VARCHAR(2000),--the name of the current item
       @path VARCHAR(2000),--the path of the current item 
       @ModifyDate DATETIME,--the date the current item last modified
       @IsFileSystem INT, --1 if the current item is part of the file system
       @IsFolder INT --1 if the current item is a file
   IF LEN(COALESCE(@Wildcard,''))<2 
       RETURN

   SELECT  @strErrorMessage = 'opening the Shell Application Object' 
   EXECUTE @hr = sp_OACreate 'Shell.Application', 
       @objShellApplication OUT 
   --now we get the folder.
   IF @HR = 0  
       SELECT  @objErrorObject = @objShellApplication, 
               @strErrorMessage = 'Getting Folder"' + @wildcard + '"', 
               @command = 'NameSpace("'+@wildcard+'")' 
   IF @HR = 0  
       EXECUTE @hr = sp_OAMethod @objShellApplication, @command, 
           @objFolder OUT
   IF @objFolder IS NULL RETURN --nothing there. Sod the error message
   --and then the number of objects in the folder
       SELECT  @objErrorObject = @objFolder, 
               @strErrorMessage = 'Getting count of Folder items in "' + @wildcard + '"', 
               @command = 'Items.Count' 
   IF @HR = 0  
       EXECUTE @hr = sp_OAMethod @objfolder, @command, 
           @count OUT
    IF @HR = 0 --now get the FolderItems collection 
        SELECT  @objErrorObject = @objFolder, 
                @strErrorMessage = ' getting folderitems',
               @command='items()'
    IF @HR = 0  
        EXECUTE @hr = sp_OAMethod @objFolder, 
            @command, @objFolderItems OUTPUT 
   SELECT @ii = 0
   WHILE @hr = 0 AND @ii< @count --iterate through the FolderItems collection
            BEGIN 
                IF @HR = 0  
                    SELECT  @objErrorObject = @objFolderItems, 
                            @strErrorMessage = ' getting folder item ' 
                                   + CAST(@ii AS VARCHAR(5)),
                           @command='item(' + CAST(@ii AS VARCHAR(5))+')'
                           --@Command='GetDetailsOf('+ cast(@ii as varchar(5))+',1)'
                IF @HR = 0  
                    EXECUTE @hr = sp_OAMethod @objFolderItems, 
                        @command, @objItem OUTPUT 

                IF @HR = 0  
                    SELECCT  @objErrorObject = @objItem, 
                            @strErrorMessage = ' getting folder item properties'
                                   + CAST(@ii AS VARCHAR(5))
                IF @HR = 0  
                    EXECUTE @hr = sp_OAMethod @objItem, 
                        'path', @path OUTPUT
                IF @HR = 0  
                    EXECUTE @hr = sp_OAMethod @objItem, 
                        'name', @name OUTPUT
                IF @HR = 0  
                    EXECUTE @hr = sp_OAMethod @objItem, 
                        'ModifyDate', @ModifyDate OUTPUT
                IF @HR = 0  
                    EXECUTE @hr = sp_OAMethod @objItem, 
                        'IsFileSystem', @IsFileSystem OUTPUT
                IF @HR = 0  
                    EXECUTE @hr = sp_OAMethod @objItem, 
                        'IsFolder', @IsFolder OUTPUT
               --and insert the properties into a table
               INSERT INTO @MyDir ([NAME], [path], ModifyDate, IsFileSystem, IsFolder)
                   SELECT @NAME, @path, @ModifyDate, @IsFileSystem, @IsFolder
               IF @HR = 0  EXECUTE sp_OADestroy @objItem 
               SELECT @ii=@ii+1
            END 
        IF @hr <> 0  
            BEGIN 
                DECLARE @Source VARCHAR(255), 
                    @Description VARCHAR(255), 
                    @Helpfile VARCHAR(255), 
                    @HelpID INT 
     
                EXECUTE sp_OAGetErrorInfo @objErrorObject, @source OUTPUT, 
                    @Description OUTPUT, @Helpfile OUTPUT, @HelpID OUTPUT 
                SELECT  @strErrorMessage = 'Error whilst ' 
                        + COALESCE(@strErrorMessage, 'doing something') + ', ' 
                        + COALESCE(@Description, '') 
                INSERT INTO @MyDir(error) SELECT  LEFT(@strErrorMessage,2000) 
            END 
        EXECUTE sp_OADestroy @objFolder 
        EXECUTE sp_OADestroy @objShellApplication

RETURN
END
/*====================================================================*/
ALTER FUNCTION [dbo].[files]
(@Wildcard VARCHAR(100),
@Subdirectories INT=0,
@details INT=1
)
/*
Usage: 
Select sum(size), count(*) 
    from dbo.files ('C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\LOG', 1,1)

*/
RETURNS @FileTable TABLE 
                   (MyID INT IDENTITY(1,1),
                   [name] VARCHAR(1000), 
                   FullPathName VARCHAR(2000),
                   [ShortPath]   VARCHAR(2000),
                   [Type] VARCHAR(100),
                   [DateCreated]  DATETIME,
                   [DateLastAccessed]    DATETIME,
                   [DateLastModified]     DATETIME,
                   [Attributes]   INT,
                   [size] BIGINT,
                   [error] VARCHAR(2000))

AS
BEGIN
DECLARE @hr INT,         --the HRESULT returned from 
       @objFileSystem INT,        --the FileSystem object
       @objFile INT,              --the File object
       @ErrorObject INT,          --the error object
       @ErrorMessage VARCHAR(255),--the potential error message
       @Path VARCHAR(5000),--
       @ShortPath VARCHAR(2000),
       @Type VARCHAR(100),
       @DateCreated DATETIME,
       @DateLastAccessed DATETIME,
       @DateLastModified DATETIME,
      @directory VARCHAR(2000),
      @MyID INT,
       @Attributes INT,
       @size BIGINT,
      @ii INT,
      @iiMax INT,
      @command VARCHAR(8000),
      @FileName VARCHAR(8000),
      @more INT

DECLARE @FileAndDirectoryList TABLE 
                   (MyID INT IDENTITY(1,1),
                   [name] VARCHAR(1000), 
                   FullPathName VARCHAR(2000),
                   [isFolder] INT,
                   [ModifyDate] DATETIME,
                   [error] VARCHAR(2000),
                   [recursed] INT DEFAULT 0
                   )


SET @more=1


INSERT INTO  @FileAndDirectoryList([name],fullPathName, [ModifyDate], IsFolder, error)
   SELECT [name], [path], [ModifyDate], IsFolder, error 
       FROM dbo.dir(@wildcard) 
       WHERE IsFileSystem =1
IF EXISTS (SELECT * FROM  @FileAndDirectoryList WHERE error IS NOT NULL) 
   RETURN
WHILE @subdirectories<>0 AND @more>0
   BEGIN
   SELECT TOP 1  @MyID= MyID
       FROM  @FileAndDirectoryList WHERE isFolder=1 AND recursed =0
   SET @more= @@rowcount
   IF @more > 0
       BEGIN
       SELECT @directory= LEFT([FullPathName],2000)
               FROM  @FileAndDirectoryList 
               WHERE MyID=@MyID 
       INSERT INTO  @FileAndDirectoryList
                   ([name],fullPathName,[ModifyDate], IsFolder, error)
           SELECT [name],[path],[ModifyDate],IsFolder,error 
               FROM dbo.dir(@directory) 
               WHERE IsFileSystem =1
       UPDATE  @FileAndDirectoryList SET recursed=1 WHERE MyID=@MyID
       END
   END
INSERT INTO @fileTable ([name],fullPathName,DateLastModified)
   SELECT  [Name], fullPathName, [ModifyDate] 
       FROM @FileAndDirectoryList WHERE isFolder=0
           OR REVERSE(fullPathName) LIKE 'piz.%'
SELECT @hr=0,@errorMessage='opening the file system object '
EXEC @hr = sp_OACreate 'Scripting.FileSystemObject',
                                       @objFileSystem OUT
SELECT @ii=MIN(MyID), @iiMax=MAX(MyID) FROM @FileTable
WHILE @hr=0 AND @ii<=@iiMax AND @Details<>0
   BEGIN
   SELECT @Filename=FullPathName FROM @fileTable 
               WHERE MyID=@ii
   IF @hr=0 
      SELECT @errorMessage='getting the attributes of '''
                                      +@Filename+'''',
          @ErrorObject=@objFileSystem
   IF @hr=0 EXEC @hr = sp_OAMethod @objFileSystem,
        'GetFile',  @objFile OUT,@Filename
   IF @hr=0 EXEC @hr = sp_OAGetProperty 
                @objFile, 'ShortPath', @ShortPath OUT
   IF @hr=0 EXEC @hr = sp_OAGetProperty 
                @objFile, 'Type', @Type OUT
   IF @hr=0 EXEC @hr = sp_OAGetProperty 
                @objFile, 'DateCreated', @DateCreated OUT
   IF @hr=0 EXEC @hr = sp_OAGetProperty 
                @objFile, 'DateLastAccessed', @DateLastAccessed OUT
   IF @hr=0 EXEC @hr = sp_OAGetProperty 
                @objFile, 'DateLastModified', @DateLastModified OUT
   IF @hr=0 EXEC @hr = sp_OAGetProperty 
                @objFile, 'Attributes', @Attributes OUT
   IF @hr=0 EXEC @hr = sp_OAGetProperty 
                @objFile, 'size', @size OUT
   IF @hr=0 
       UPDATE @FileTable 
           SET [ShortPath]= @ShortPath,
               [Type]= @Type,
               [DateCreated]=  @DateCreated ,
               [DateLastAccessed]=     @DateLastAccessed,
               [DateLastModiified]=     @DateLastModified,
               [Attributes]=   @Attributes,
               [size]= @size
           WHERE MyID=@ii
   SELECT @ii=@ii+1
   END
IF @hr<>0
       BEGIN
       DECLARE 
               @Source VARCHAR(255),
               @Description VARCHAR(255),
               @Helpfile VARCHAR(255),
               @HelpID INT
       
       EXECUTE sp_OAGetErrorInfo  @errorObject, 
               @source OUTPUT,@Description OUTPUT,
                               @Helpfile OUTPUT,@HelpID OUTPUT

       SELECT @ErrorMessage='Error whilst '
                               +@Errormessage+', '
                               +@Description
       INSERT INTO @FileTable (error) SELECT  LEFT(@ErrorMessage,2000) 
       END
EXEC sp_OADestroy @objFileSystem
EXEC sp_OADestroy @objFile
RETURN
/*==============================================*/
IF OBJECT_ID (N'dbo.spLoadTextFromAFile') IS NOT NULL
   DROP PROCEDURE dbo.spLoadTextFromAFile
GO
CREATE PROCEDURE spLoadTextFromAFile
   @Filename VARCHAR(255),
   @Unicode INT = 0
--spLoadTextFromAFile 'C:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\LOG\ERRORLOG.6', @Unicode=1
AS 
   SET NOCOUNT ON
   DECLARE @MySpecialTempTable VARCHAR(255)
   DECLARE @Command NVARCHAR(4000)
   DECLARE @RESULT INT

--firstly we create a global temp table with a unique name
   SELECT   @MySpecialTempTable = '##temp' 
       + CONVERT(VARCHAR(12), CONVERT(INT, RAND() * 1000000))
--then we create it using dynamic SQL, 
--
   SELECT   @Command = 'create table [' 
       + @MySpecialTempTable + '] (MyLine '
       + CASE WHEN @unicode <>0 THEN 'N' ELSE '' END +'varchar(MAX))
 '
   EXECUTE sp_ExecuteSQL @command

   SELECT   @command = 'bulk insert [' 
       + @MySpecialTempTable + '] from' + ' ''' 
       + REPLACE(@Filename, '"', '') + '''' 
       + ' with (FIELDTERMINATOR=''|~||''' + ',ROWTERMINATOR = ''
'''        + CASE WHEN @unicode <>0 THEN ', DATAFILETYPE=''widechar'''ELSE '' END
       + ')'
-- import the data
   EXEC (@Command)
   EXECUTE ('Select * from ' + @MySpecialTempTable)
   EXECUTE ('Drop table ' + @MySpecialTempTable)
GO
